Package: funHDDC
Type: Package
Title: Univariate and Multivariate Model-Based Clustering in
        Group-Specific Functional Subspaces
Version: 2.3.1.1
Date: 2026-05-05
Authors@R: c(person(given = "A",
                family = "Schmutz",
                role = "aut"),
        person(given = c("J.", "Jacques", "&", "C."),
                family = "Bouveyron",
                role = "aut"),
        person(given = "Julien",
                family = "Jacques",
                role = "cre",
                email = "julien.jacques@univ-lyon2.fr"))
Maintainer: Julien Jacques <julien.jacques@univ-lyon2.fr>
Depends: MASS,fda, R (>= 3.1.0)
Description: The funHDDC algorithm  allows to cluster functional univariate (Bouveyron and Jacques, 2011, <doi:10.1007/s11634-011-0095-6>) or multivariate data (Schmutz et al., 2018) by modeling each group within a specific functional subspace.
License: GPL-2
Encoding: UTF-8
LazyData: false
RoxygenNote: 6.0.1
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-05 13:57:11 UTC; siyanasran
Repository: CRAN
Date/Publication: 2026-05-08 15:20:02 UTC
Author: A Schmutz [aut],
  J. Jacques & C. Bouveyron [aut],
  Julien Jacques [cre]
Built: R 4.6.0; ; 2026-05-08 16:49:52 UTC; unix
